/* This file was generated by syncqt. */
#ifndef QT_QTQUICK3DPHYSICS_VERSION_H
#define QT_QTQUICK3DPHYSICS_VERSION_H

#define QTQUICK3DPHYSICS_VERSION_STR "6.12.0"

#define QTQUICK3DPHYSICS_VERSION 0x0601200

#endif // QT_QTQUICK3DPHYSICS_VERSION_H
